
public class ConvertIOData {

	// for convert() and related methods
	private String textStr = null;
	private StringBuffer textBuf = null;
	private String hexStr = null;
	private StringBuffer hexBuf = null;
	private boolean keepNewlines = true;

	// for binarize() and related methods
	private int totLength;
	
	public ConvertIOData() {
		this(false);
	}
	
	public ConvertIOData(boolean tossOutNewlines) {
		keepNewlines = !tossOutNewlines;
	}

	public synchronized void convert(byte[] data, int dataLength) {
		textBuf = new StringBuffer("");
		hexBuf = new StringBuffer("");
		String hexCharStr;
		int i;
		for ( i = 0 ; i < dataLength ; i++ ) {
			short t = (short)data[i];
			if(t < 0) {
				t += 256;
			}
			//textBuf.append((char)data[i]);
			if (t == '\n') {
				if (keepNewlines) {
					textBuf.append('\n');
				} else {
					textBuf.append(' ');
				}
			} else if (t < 0x20) {
				// unprintable
				textBuf.append(' '); 
			} else {
				// printable, although it may not display
				textBuf.append((char)t);
			}
			//hexBuf.append((short)data[i]);
			hexCharStr = StrFormat.getFormatted("%x",t);
			hexCharStr = StrFormat.padZero(hexCharStr,2);
			hexBuf.append(hexCharStr);
			hexBuf.append(' ');
			//System.out.println(data[i]);
			//String fmtstr = String.format("%d\n",data[i]);
		}
		textStr = new String(textBuf);
		hexStr = new String(hexBuf);
	}

	// return a result of convert()
	public synchronized String getText() {
		return textStr;
	}

	// return a result of convert()
	public synchronized String getHex() {
		return hexStr;
	}

	// binarize (that means convert backslash escape sequences into special characters)
	public byte[] binarize(byte[] bufIn) {
		byte[] bufNew = new byte[bufIn.length];
		int i;
		boolean readingEscapeSequence = false;
		boolean readingHexValue = false;
		byte hexDigitCount = 0;
		byte hexValue = 0;
		totLength = 0;
		for ( i = 0 ; i < bufIn.length ; i++ ) {
			//System.out.print("\ni=" + i + ", ");
			if (readingHexValue) {
				hexDigitCount++;
				// escape sequence indicating a hex value is being read
				if ( bufIn[i] >= '0' && bufIn[i] <= '9' ) {
					hexValue += bufIn[i] - '0';
				} else if ( bufIn[i] >= 'a' && bufIn[i] <= 'f' ) {
					hexValue += bufIn[i] + 0xa - 'a';
				} else {
					readingHexValue = false;
					readingEscapeSequence = false;
					hexValue = 0;
					hexDigitCount = 0;
				}
				if ( hexDigitCount == 1 ) {
					hexValue *= 0x10;
				} else {
					bufNew[totLength] = hexValue;
					totLength++;
					readingHexValue = false;
					readingEscapeSequence = false;
					hexValue = 0;
					hexDigitCount = 0;
				}
			} else if (readingEscapeSequence) {
				// a backslash was seen and we are reading a sequence after it
				switch(bufIn[i]) {
					case '\\':
						bufNew[totLength] = '\\';
						totLength++;
						readingEscapeSequence = false;
						break;
					case 'n':
						bufNew[totLength] = '\n';
						totLength++;
						readingEscapeSequence = false;
						break;
					case 'r':
						bufNew[totLength] = '\r';
						totLength++;
						readingEscapeSequence = false;
						break;
					case 'b':
						bufNew[totLength] = '\b';
						totLength++;
						readingEscapeSequence = false;
						break;
					case 't':
						bufNew[totLength] = '\t';
						totLength++;
						readingEscapeSequence = false;
						break;
					case 'f':
						bufNew[totLength] = '\f';
						totLength++;
						readingEscapeSequence = false;
						break;
					case 'x':
						readingHexValue = true;
						break;
					default:
						readingEscapeSequence = false;
				}
			} else {
				// we are not currently reading an escape sequence
				if (bufIn[i] == '\\') {
					// on the next character we are reading an escape sequence
					readingEscapeSequence = true;
				} else {
					// ordinary character
					bufNew[totLength] = bufIn[i];
					totLength++;
					//System.out.print("ordinary character");
				}
			}
		}
		return bufNew;
	}

	public int lastBinarizeLength() {
		return totLength;
	}

}
